# -*- coding: utf-8 -*-

DEFAULT_TOKEN = b"ThisIsAnInsecurePublicTokenDontUseIt"

DEFAULT_SALT = "Quisque vulputate nibh eros, vehicula sollicitudin neque pellentesque quis"

DEFAULT_LOGIN_PATH = 'ajaxlogin'

DEFAULT_NEW_TOKEN_PATH = 'ajaxtoken'



# -*- coding: utf-8 -*-

from py4web import request, abort
from py4web.core import Fixture
from base64 import b64decode

from . import settings

class IsValidToken(Fixture):
    """docstring for IsValidToken."""

    def __init__(self, token=settings.DEFAULT_TOKEN):
        super(IsValidToken, self).__init__()
        self.TOKEN = token

    @staticmethod
    def __get_token():
        encoded = request.headers['Authorization'].replace('Bearer ', '')
        return b64decode(encoded)

    def on_request(self):

        token = self.__get_token()
        if token!=self.TOKEN:
            abort(401, 'Unrecognized token')
        # elif self.TOKEN==settings.DEFAULT_TOKEN and not request.urlparts.netloc.startswith('localhost'):
        #     abort(403)
            # raise HTTP(403)



# -*- coding: utf-8 -*-

from mptools.frameworks.py4web.controller import CORS
from py4web.utils.url_signer import URLSigner
from py4web import action, request, URL, redirect
from py4web.utils.factories import ActionFactory

from . import settings
from .tools import IsValidToken

from .common import LoginChecker

class FixtureChain(object):
    """docstring for FixtureChain."""

    def __init__(self, *fixtures):
        super(FixtureChain, self).__init__()
        self.fixtures = fixtures

    def __call__(self, *fixtures):
        return self.fixtures+fixtures

    def push(self, *fixtures):
        self.fixtures = fixtures+self.fixtures


def enable(auth, origin='*', token_lifespan=10, use_auth_model=False, test=False):
    """ """

    loginchecker = LoginChecker(auth, origin=origin, use_auth_model=use_auth_model)
    urlsigner = URLSigner(variables_to_sign=['id'], lifespan=token_lifespan)

    # TODO: Support for different method other than PUT

    @action(settings.DEFAULT_NEW_TOKEN_PATH, method=['PUT'])
    @action.uses(CORS(origin=origin), IsValidToken(), urlsigner)
    def intercom():
        user_id = request.json['user_id']
        dest = request.json.get('dest', settings.DEFAULT_LOGIN_PATH)
        return dict(url=URL(dest, vars=dict(
            hashed_user_id = loginchecker.encode(user_id)
        ), signer=urlsigner, scheme='https'))

    @action(settings.DEFAULT_LOGIN_PATH, method=['GET', 'POST'])
    @action.uses(urlsigner.verify(), loginchecker, CORS(origin=origin, session=auth.session))
    def ajaxlogin():
        return dict(
            {'user_id': auth.user_id} if test else {},
            message = 'OK' if auth.user_id else 'ANONYMOUS',
        )

    if not test is False:

        @action('testcall', method=['OPTIONS'])
        @action.uses(CORS(origin=origin), auth.session)
        def _():
            return dict()

        @action('testcall', method=['GET', 'POST', 'OPTIONS'])
        @action.uses(auth.session, CORS(origin=origin, session=auth.session), auth.user)
        def _():
            return dict(user_id=auth.user_id)

    return ActionFactory(loginchecker, urlsigner, CORS(origin=origin, session=auth.session))



# -*- coding: utf-8 -*-

from . import settings
from hashids import Hashids
from datetime import datetime, timezone
import calendar, time
from py4web.core import Fixture
from mptools.frameworks.py4web.controller import web_call
from py4web import request, abort, HTTP
from urllib.parse import urlparse

class IdHasher(object):
    """docstring for IdHasher."""

    def __init__(self):
        super(IdHasher, self).__init__()
        self.idhasher = Hashids(salt=settings.DEFAULT_SALT)

    def encode(self, id):
        timestamp = int(datetime.now(timezone.utc).timestamp()*100)
        return self.idhasher.encode(int(id), timestamp)

    def decode(self, hased_id):
        decoded = self.idhasher.decode(hased_id)
        if decoded:
            return decoded[0]
        else:
            abort(401, "Wrong code")

idhasher = IdHasher()

class LoginChecker(Fixture):
    """docstring for LoginChecker."""

    def __init__(self, auth, origin, use_auth_model=False):
        super(LoginChecker, self).__init__()
        self.auth = auth
        self.use_auth_model = use_auth_model
        self.origin = origin

    encode = lambda self, id: idhasher.encode(id)
    decode = lambda self, hashed_id: idhasher.decode(hashed_id)

    def __auth_login(self, user_id, email):
        if self.use_auth_model:
            # Login with DB
            user = {}
            origin = urlparse(request.headers.environ.get("HTTP_ORIGIN", self.origin)).hostname
            user["sso_id"] = f"{origin}:{user_id}"
            user['username'] = user["sso_id"]
            user['email'] = f"user_{user_id}@{origin}"
            data = self.auth.get_or_register_user(user)
            assert data
            self.auth.store_user_in_session(data['id'])
        else:
            # Login without DB
            self.auth.store_user_in_session(user_id)
        assert self.auth.user_id

    def __call__(self, hashed_user_id, email=None, **__):
        """ Decode the remote user_id and use it for login """

        # The user could be logged in but the session could be expired
        # Courtesy of: https://github.com/web2py/py4web/blob/master/py4web/utils/auth.py#L95
        if not self.auth.user_id is None:
            activity = self.auth.session.get("recent_activity")
            time_now = calendar.timegm(time.gmtime())
            if self.auth.param.login_expiration_time and activity:
                if time_now - activity > self.auth.param.login_expiration_time:
                    self.auth.session.clear()

        if self.auth.user_id is None:
            user_id = self.decode(hashed_user_id)
            self.__auth_login(user_id, email=email)

    def on_request(self):
        web_call(self)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ajaxlogin",
    version="0.0.2",
    author="Manuele Pesenti",
    author_email="manuele@inventati.org",
    description="A lightweight ajax login solution py4web based",
    long_description=long_description,
    long_description_content_type="text/markdown",
    # url="https://github.com/pypa/sampleproject",
    # project_urls={
    #     "Bug Tracker": "https://github.com/pypa/sampleproject/issues",
    # },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
    install_requires=[
        "py4web>=1.20210426.1",
        "mptools>=1.0.6"
    ],
)
